/*
 * Copyright (C) 2006 uguu@users.sourceforge.jp, All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. Neither the name of Clarkware Consulting, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without prior written permission. For written
 *       permission, please contact clarkware@clarkware.com.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * CLARKWARE CONSULTING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN  ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package jp.sourceforge.expression_computer.node;

import jp.sourceforge.expression_computer.CompileContext;
import jp.sourceforge.expression_computer.Node;
import jp.sourceforge.expression_computer.command.PushStackCommand;
import jp.sourceforge.expression_computer.type.IntegerLiteral;
import jp.sourceforge.expression_computer.util.Validator;

/**
 * <p>
 * IntegerLiteralを表すノードです。
 * </p>
 * 
 * @author uguu@users.sourceforge.jp
 */
public final class IntegerLiteralNode extends AbstractNode implements OperandNode {

    private static final int RADIX_16 = 16;

    private static final int RADIX_8  = 8;

    private String           value;

    /**
     * <p>
     * インスタンスを初期化します。
     * </p>
     * 
     * @param value
     *            値。<br>
     *            nullの場合、{@link NullPointerException}例外をスローします。<br>
     *            整数値の形式ではない場合、{@link IllegalArgumentException}例外をスローします。
     */
    public IntegerLiteralNode(String value) {
        if (value == null) {
            throw new NullPointerException("valueがnullです。");
        }
        if (!Validator.isIntegerLiteral(value)) {
            throw new IllegalArgumentException("valueが整数値の形式ではありません。");
        }
        this.value = value;
    }

    /**
     * {@inheritDoc}
     */
    public void compile(CompileContext context) {
        long value;
        if (this.value.startsWith("0x")) {
            value = Long.parseLong(this.value.substring("0x".length()), IntegerLiteralNode.RADIX_16);
        } else if (this.value.startsWith("0") && this.value.length() > 1) {
            value = Long.parseLong(this.value.substring("0".length()), IntegerLiteralNode.RADIX_8);
        } else {
            value = Long.parseLong(this.value);
        }

        context.getCommandList().add(new PushStackCommand(new IntegerLiteral(value)));
    }

    /**
     * {@inheritDoc}
     */
    public Node[] getChildren() {
        return new Node[0];
    }

    /**
     * <p>
     * 値を返します。
     * </p>
     * 
     * @return 値。
     */
    public String getValue() {
        return this.value;
    }

    /**
     * {@inheritDoc}
     */
    public String toString() {
        return this.getClass().getName() + "[" + this.value + "]";
    }

}
